# Copyright (C) 2007 by Intevation GmbH
# Authors:
# Bernhard Herzog <bh@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file GPL.txt coming with the software for details.

"""Support code for tests involving osaas.http.httpserver"""

import logging
import unittest
import httplib
import re
import time
from math import floor
import StringIO


import osaas.http.httpserver as httpserver


class HTTPRequestHandler(httpserver.HTTPRequestHandler):

    def do_GET(self):
        """Handle a GET request"""
        for path, headers, data in self.server.contents:
            if path == self.path:
                self.send_response(200)
                for header, value in headers:
                    self.send_header(header, value)
                self.send_header("Content-Length", len(data))
                self.end_headers()
                self.wfile.write(data)
                return
        else:
            self.send_error(404)


class HTTPServer(httpserver.HTTPServer):

    def __init__(self, contents, server_address=("127.0.0.1", 0),
                 RequestHandlerClass=HTTPRequestHandler, **kw):
        self.contents = contents
        httpserver.HTTPServer.__init__(self, server_address,
                                       RequestHandlerClass, **kw)


class LoggerMixin:

    def new_logger(self, logger_name, format="%(message)s"):
        """Create a new logger with some default configuration.

        The return value is the StringIO object used as the stream into
        which the logger writes.
        """
        logger = logging.getLogger(logger_name)
        logger.setLevel(logging.DEBUG)
        logger.propagate = False
        stream = StringIO.StringIO()
        hdlr = logging.StreamHandler(stream)
        hdlr.setFormatter(logging.Formatter(format))
        logger.addHandler(hdlr)
        return stream


class ServerTest(unittest.TestCase, LoggerMixin):

    access_logger_name = "test.access"
    error_logger_name = "test.error"

    def setUp(self):
        self.access_log_stream = self.new_logger(self.access_logger_name)
        self.error_log_stream = self.new_logger(self.error_logger_name)

        server = self.create_server(("127.0.0.1", 0),
                                    access_logger_name=self.access_logger_name,
                                    error_logger_name=self.error_logger_name)
        server.test_case = self
        self.server = httpserver.ServerThread(server)
        self.server.start(daemon=True)

        self.test_start_time = time.time()

    def create_server(self, *args, **kw):
        return httpserver.HTTPServer(*args, **kw)

    def tearDown(self):
        self.server.stop()

    def reset_access_log(self):
        self.access_log_stream.truncate(0)

    def check_log(self, expected_lines):
        log_lines = self.access_log_stream.getvalue().splitlines()

        for idx in range(len(log_lines)):
            line = log_lines[idx]

            # extract the timestamp from the line so that we can check
            # the contents of the line without the ever changing time
            # stamp and check the timestamp value separately
            match = re.search(r"\[([^]]+)\]", line)
            if match:
                line = line[:match.start(1)] + line[match.end(1):]

                # Check time stamps.  Round both time stamps down to the
                # nearest second. Otherwise start_time might actually be
                # larger than timestamp because timestamp only has 1s
                # accuracy
                # FIXME: enforce "C" locale for strptime
                timestamp = time.mktime(time.strptime(match.group(1),
                                                      "%d/%b/%Y %H:%M:%S"))
                self.failUnless(floor(timestamp) >= floor(self.test_start_time))

            log_lines[idx] = line

        self.assertEquals(log_lines, expected_lines)


    def run_simple_http_test(self, path, status, access_log=None,
                             method="GET", extra_headers=(), body=None):
        self.reset_access_log()
        http = httplib.HTTPConnection("localhost", self.server.server_port)
        http.putrequest(method, path)
        for header, value in extra_headers:
            http.putheader(header, value)
        http.endheaders()
        if body is not None:
            http.send(body)
        response = http.getresponse()
        self.assertEquals(response.status, status)

        if access_log is None:
            access_log = ['localhost - - [] "%s %s HTTP/1.1" %d -'
                          % (method, path, status)]
        self.check_log(access_log)
        return response.read()
