# Copyright (C) 2007 by Intevation GmbH
# Authors:
# Bernhard Herzog <bh@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file GPL.txt coming with the software for details.

"""Tests for osaas.http.run"""

import sys
import unittest
import optparse
import traceback

from osaas.http.run import ProgramWithOptions, OptionStack


class TestProgramWithOptions(unittest.TestCase):

    def test_default_options(self):
        prog = ProgramWithOptions()
        opts, rest = prog.parse_options([])
        self.assertEquals(opts.config_file, None)

    def test_options(self):
        prog = ProgramWithOptions()
        opts, rest = prog.parse_options(["--config-file=/etc/myconfig"])
        self.assertEquals(opts.config_file, "/etc/myconfig")

    def test_adding_options(self):
        class Program(ProgramWithOptions):
            def create_option_parser(self, **kw):
                parser = ProgramWithOptions.create_option_parser(self, **kw)
                parser.add_option("--num-threads", type="int")
                parser.set_defaults(num_threads=5)
                return parser

        # defaults
        prog = Program()
        opts, rest = prog.parse_options([])
        self.assertEquals(opts.config_file, None)
        self.assertEquals(opts.num_threads, 5)

        # some options supplied
        prog = Program()
        opts, rest = prog.parse_options(["--config-file=/etc/config2",
                                         "--num-threads=10"])
        self.assertEquals(opts.config_file, "/etc/config2")
        self.assertEquals(opts.num_threads, 10)

    def test_read_config_file_method(self):
        class Program(ProgramWithOptions):
            def create_option_parser(self, **kw):
                parser = ProgramWithOptions.create_option_parser(self, **kw)
                parser.add_option("--num-threads", type="int")
                parser.add_option("--error-log")
                parser.add_option("--port", type="int")
                parser.set_defaults(num_threads=5, error_log="/var/log/error",
                                    port=7777)
                return parser
            def read_config_file(self, opts):
                opts.set_file_option("num_threads", 10)
                opts.set_file_option("port", 9988)

        prog = Program()
        opts, rest = prog.parse_options(["--port=8899"])

        # The port has been given on the command line, so that value
        # should win:
        self.assertEquals(opts.port, 8899)

        # The num_threads option has been set in read_config_file but
        # not on the command line, so the value from read_config_file
        # wins:
        self.assertEquals(opts.num_threads, 10)

        # The error_log has only been set in the defaults (with the
        # default default value of "/var/log/error").
        self.assertEquals(opts.error_log, "/var/log/error")

    def test_optparse_option_class(self):
        class MyOption(optparse.Option):
            ATTRS = optparse.Option.ATTRS[:]
            ATTRS.extend(["extra_arg"])

        class Program(ProgramWithOptions):
            optparse_option_class = MyOption
            def create_option_parser(self, **kw):
                parser = ProgramWithOptions.create_option_parser(self, **kw)
                # the default option class will raise an error when an
                # unknown argument like "extra_arg" is passed.
                parser.add_option("--some-option", extra_arg=True)
                return parser

        prog = Program()
        try:
            prog.parse_options([])
        except:
            self.fail("Unexpected exception when parsing options:\n%s"
                      % "".join(traceback.format_exception(*sys.exc_info())))

class TestOptionStack(unittest.TestCase):

    def test(self):
        parser = optparse.OptionParser()
        parser.add_option("--port", type="int")
        parser.add_option("--config-file")
        parser.add_option("--log-file")
        parser.set_defaults(port=1234)

        default_opts = parser.get_default_values()
        configfile_opts = optparse.Values()
        commandline_opts, rest = parser.parse_args(["--config-file=/etc/cfg"],
                                                   optparse.Values())
        opts = OptionStack([], default_opts, configfile_opts, commandline_opts)

        opts.set_file_option("log_file", "/var/log/my.log")

        self.assertEquals(opts.port, 1234)
        self.assertEquals(opts.config_file, "/etc/cfg")
        self.assertEquals(opts.log_file, "/var/log/my.log")
